#!/usr/bin/env python3

import os
import glob
from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

scripts = glob.glob('bin/*')

with open(os.path.join('arts_tools', '__version__.py')) as version_file:
    version = {}
    exec(version_file.read(), version)
    project_version = version['__version__']

setup(name='arts_tools',
      version=project_version,
      description='Tools to read or process ARTS data',
      long_description=readme,
      long_description_content_type="text/markdown",
      url='http://github.com/loostrum/arts_tools',
      author='Leon Oostrum',
      author_email='l.oostrum@esciencecenter.nl',
      license='Apache2.0',
      zip_safe=False,
      include_package_data=True,
      packages=find_packages(),
      install_requires=['numpy>=1.17',
                        'astropy',
                        'psrqpy',
                        'tqdm'],
      extras_require={'irods': ['python-irodsclient']},
      entry_points={'console_scripts':
                    ['arts_fix_fits_from_before_20200408=arts_tools.fits.fix_file_from_before_20200408:main',
                     'arts_find_pulsars_in_field=arts_tools.pulsars.find_pulsars:main',
                     'arts_download_from_alta=arts_tools.archive.download_from_alta:main',
                     'arts_psrdada_iquv_to_fits=arts_tools.iquv.psrdada_to_fits:main']},
      scripts=scripts,
      classifiers=['License :: OSI Approved :: Apache Software License',
                   'Programming Language :: Python :: 3',
                   'Operating System :: OS Independent'],
      python_requires='>=3.6'
      )
